//
//  SC01Person.m
//  PrimerSample
//
//  Created by Jp LaFond on 3/19/12.
//  Copyright (c) 2012 Jp4Mobile. All rights reserved.
//

#import "SC01Person.h"

// Optional way to define some private methods for this class that cannot be called by 
// anyone else.
// By defining them before hand, you can get the IDE to do some sanity checking for you
// to make sure that your implementation of the class is defined fully.
@interface SC01Person ()

@property (nonatomic, assign) NSInteger privateID;

- (void) logData:(id)inputClass;

@end

@implementation SC01Person

// Typically, I like to use a specific ivar name, so that we can differentiate them from
// the class variables
@synthesize firstName  = _firstName;
@synthesize lastName   = _lastName;
@synthesize huidNumber = _huidNumber;
@synthesize privateID  = _privateID;

// Our methods
// As we're using ARC and don't need to worry about releasing and deallocating things, we can ignore it.

// Our initer with all the particulars
- (id) initWithFirstName:(NSString *)firstName LastName:(NSString *)lastName HUID:(NSInteger)huidNumber {
    // Self is similar to this in Java.
    // We've just does the standard init from the parent NSObject class
    self = [super init];
    
    // Now we populate it with our own stuff
    //    We can either use the direct assign (nice, simple and clear)
    self.firstName  = firstName;
    //    Or we use the generated setter that is actually called with the direct assign
    [self setLastName:lastName];
    //    The other way of doing this is to use the ivar directly
    //        THIS IS NOT A GOOD IDEA. Always use the getters/setters in good programming 
    //        practices
    _huidNumber = huidNumber;
    self.privateID = -1 * huidNumber;
    
    // Let's log it
    [self logData:self];
    // Also logging a print constant to illustrate why the class check is a good idea
    [self logData:@"String Constant"];
    
    // We've set up everything, we send self on up to the caller
    return self;
}

// Our private helper class
- (void) logData:(id)inputClass {
    // Don't forget that every NSString constant needs an @
    NSLog(@"logData: (%@)", inputClass);
    
    // Check to see if we're using the right class
    //    Always a good idea to make sure we've got the right class
    if ([inputClass isKindOfClass:[SC01Person class]]) {
        // Log the particulars
        SC01Person * tmpPerson = (SC01Person *)inputClass;
        //      Much like with the assignment, there are two ways to get the data
        NSLog(@"logData:   firstName : %@", [tmpPerson firstName]);
        NSLog(@"logData:   lastName  : %@", tmpPerson.lastName);
        NSLog(@"logData:   huidNumber: %d", tmpPerson.huidNumber);
        NSLog(@"logData:   privateID : %d", tmpPerson.privateID);
    }
}

// Public access to logData
- (void) printMe:(id)something {
    [self logData:something];
}

@end
